#! /bin/bash -e
# This script installs RightLink assuming it has been downloaded and unpacked. It is
# called at the end of the rightlink.boot.sh, rightlink.enable.sh and rightlink.install.sh
# scripts, which also may set up dependencies and credentials in addition to
# installing RightLink itself.

autostart_service=0

show_help() {
  cat <<EOT
Usage: ${0##*/} [-options...]
Installs/Upgrades RightLink on an an image.
  -s    start rightlink service at end of this script
  -n    skip installation of RightScale network configuration scripts for RCA-V
  -h    show this help
  -v    verbose mode
EOT
}

no_networking_scripts=0

while getopts "hnvs" opt; do
  case "$opt" in
    h) show_help; exit 0 ;;
    v) set -x ;;
    s) autostart_service=1 ;;
    n) no_networking_scripts=1 ;;
    '?') show_help >&2; exit 1 ;;
  esac
done

# Determine if /usr/local/bin is read-only. If so, use /opt/bin to install executables
mkdir -p /usr/local/bin || true
[ -w /usr/local/bin ] && BIN_DIR=/usr/local/bin || BIN_DIR=/opt/bin

# Where the package got downloaded and expanded
TMP=/tmp/rightlink
# Where to put the executables
RL=$BIN_DIR/rightlink
RSC=$BIN_DIR/rsc
# Where to put the userdata info with the params allowing to connect to RS platform
ID=/var/lib/rightscale-identity

# the identity file gets created either by cloud-init or in the enable script, ensure
# that it's only readable by privileged processes
if [[ -e $ID ]]; then
  chmod 0600 $ID
fi

if [[ ! -d $TMP ]]; then
  echo "Expected RightLink package to be expanded into directory $TMP, but it's missing."
  exit 1
fi
cd $TMP

if [[ ! $PATH =~ sbin ]]; then
  export PATH=$PATH:/sbin:/usr/sbin
fi

# Install the RightScale Network Agent for RCA-V, if we're on vsphere cloud. We do this first
# in case the installation fails.
if which vmtoolsd >/dev/null 2>&1; then
  if [[ "$no_networking_scripts" == "1" ]]; then
    if [[ -e /usr/local/share/rightnetwork/uninstall.sh ]]; then
      echo "Removing RightScale network configuration scripts for RCA-V"
      chmod +x /usr/local/share/rightnetwork/uninstall.sh
      /usr/local/share/rightnetwork/uninstall.sh
    else
      echo "Skipping installation of RightScale networking configuration scripts for RCA-V"
    fi
  else
    echo "Installing RightScale networking configuration scripts for RCA-V"
    BASE_DIR=`dirname $0`
    chmod a+x $BASE_DIR/rightnetwork/scripts/install.sh
    $BASE_DIR/rightnetwork/scripts/install.sh
  fi
fi

# Create rightlink service user for rightlink binary.
if [[ `id -u` != '0' ]]; then
  echo "ERROR: must be root user to install RightLink" >&2
  exit 1
elif [[ ! -x /usr/sbin/useradd ]]; then
  echo "ERROR: useradd doesn't exist, can't create rightlink user" >&2
  exit 1
fi

if ! grep sudoers.d /etc/sudoers >/dev/null 2>&1; then
  echo "#includedir /etc/sudoers.d" >> /etc/sudoers
  mkdir -p /etc/sudoers.d
  chmod 0750 /etc/sudoers.d
fi

echo "Creating 'rightlink' service user"

if id -u rightlink >/dev/null 2>&1; then
  echo "'rightlink' user already exists"
else
  useradd -c "rightlink service" -m -d /home/rightlink -U -s /bin/false rightlink
  cat > /etc/sudoers.d/90-rightscale <<-EOF
# Rightlink service startup requires these directives:
Defaults:root !requiretty
Defaults:rightlink !requiretty
Defaults:rightlink !env_reset
root ALL=(ALL) SETENV:ALL
# Blanket permissions. If you wish to remove the NOPASSWD:ALL line and tighten
# permissions, you must add in permissions for lifecycle management (reboot/terminate)
# via /sbin/init. The single line below would allow this:
# rightlink ALL=(root) NOPASSWD:/sbin/init
rightlink ALL=(ALL) SETENV:NOPASSWD:ALL
EOF
fi
chmod 0440 /etc/sudoers.d/90-rightscale
# If installed in certain contexts (loopback filesystem) these may be created
# with the wrong context (home_root_t instead of user_home_t)
if which restorecon >/dev/null 2>&1; then
  restorecon -RFv /home/rightlink || true
fi
echo "User 'rightlink' created with unrestricted sudo permissions"

# Install RSC command line tool
mkdir -p ${RSC%/*}
# We may have moved rsc already depending on what called this script.
cp rsc $RSC
chmod +x $RSC
echo "$($RSC --version) command line API tool installed."

# Put the binary into a bin directory
mkdir -p ${RL%/*}
cp rightlink $RL
chmod 0755 $RL

# Setup spool dir to be owned by rightlink
mkdir -p /var/spool/rightlink
chown -R rightlink:rightlink /var/spool/rightlink
chmod 700 /var/spool/rightlink

# Setup location for data files
mkdir -p /var/lib/rightlink
chown -R rightlink:rightlink /var/lib/rightlink
chmod 755 /var/lib/rightlink

# Determine whether we're upstart, systemd, or sysv-init based and plop the right config down
if [[ `/sbin/init --version 2>/dev/null` =~ upstart ]]; then
  echo "Using upstart"
  cp rightlink.conf /etc/init
  if [[ -f /etc/init/cloud-run-user-script.conf ]]; then
    # Start after user scripts run
    sed -i -e 's/cloud-final/cloud-run-user-script/' /etc/init/rightlink.conf
  elif [[ ! -f /etc/init/cloud-final.conf ]]; then
    # RHEL6/CentOS6 use sysv init for cloud-init, thus we can't start on "stopped cloud-final"
    sed -i -e 's/cloud-final/rc/' /etc/init/rightlink.conf
  fi
  if [[ "$autostart_service" == "1" ]]; then
    start rightlink
  fi
elif [[ `systemctl 2>/dev/null` =~ -\.mount ]] || [[ `readlink /sbin/init` =~ systemd ]]; then
  echo "Using systemd"
  # CoreOS installs binary at /opt/bin/rightlink. Check if in /opt/bin and update rightlink.service.
  if [[ -f /opt/bin/rightlink ]]; then
    sed -i -e 's#/usr/local/bin/rightlink#/opt/bin/rightlink#' rightlink.service
  fi
  cp rightlink.service /etc/systemd/system
  # daemon-reload here fixes a bug when rewrapping an enabled instance on systemd based systems. Systemd gets 
  # confused by the new file coming in with the same name as the old one and will give a permission
  # denied error.
  systemctl daemon-reload
  systemctl enable rightlink
  if [[ "$autostart_service" == "1" ]]; then
    systemctl start rightlink --no-block
  fi
elif [[ -x /usr/sbin/update-rc.d ]] || [[ -x /sbin/chkconfig ]]; then
  echo "Using sysv-init"
  cp rightlink.init /etc/init.d/rightlink
  if [[ -f /etc/init.d/cloud-init-user-scripts ]]; then
    sed -i 's/cloud-final/cloud-init-user-scripts/' /etc/init.d/rightlink
  fi
  if [[ -f /etc/init.d/cloud-final ]]; then
    # Debian 7.7 makes cloud-final require $all which makes it impossible to start RightLink after it
    sed -i '/Required-Start: *\$all/s/\$all *//' /etc/init.d/cloud-final
  fi
  if [[ -x /usr/sbin/update-rc.d ]]; then
    /usr/sbin/update-rc.d rightlink defaults
  elif [[ -x /sbin/chkconfig ]]; then
    /sbin/chkconfig --add rightlink
  fi
  if [[ "$autostart_service" == "1" ]]; then
    /etc/init.d/rightlink start
  fi
else
  if [[ "$autostart_service" == "0" ]]; then
    echo "Unable to determine init system, cannot install RightLink"
    exit 1
  fi
  # Create /var/run directory
  mkdir -p /var/run/rightlink
  chown -R rightlink:rightlink /var/run/rightlink
  chmod 700 /var/run/rightlink

  echo "Init system undetermined: starting RightLink in the background."
  . <(sed '/^export/!s/^/export /' $ID)
  $RL </dev/null &
  disown -a
fi

echo "`date` RightLink 10.6.4 install complete."
