#! /bin/bash

# This script disables a "RightScale server" that had previously been a running instance and
# was enabled into a "RightScale server" using the rightlink.enable.sh script.
# It does this in 5 steps:
# 1) removes RightLink from the instance
# 2) removes the rightlink user and group
# 3) removes any startup scripts that were placed by RightLink
# 4) disable the server by making a disable call to the API
# 5) removes managed login configuration and files
#
# This script is deliberately written in bash because (a) it works on all linux distros, and
# (b) almost everyone can figure out how to make modifications.
#
# For detailed instructions, see http://docs.rightscale.com/rl10/reference/rl10_disable_enabled_server.html

# Old name and location of RightLink
RLL=/usr/local/bin/rightlinklite

# Rename old log file if it exists and move secret file so RSC can make API Proxy calls
if [[ $($RLL --version 2>/dev/null) =~ (10.0.rc0|10.0.rc1|10.0.rc2) ]]; then
  if [[ -f /var/log/rightlinklite.log ]]; then
    mv -v /var/log/rightlinklite.log /var/log/rightlink.log
  fi
  # The secret file is moved and renamed so that RSC can correctly make RL10 API proxy calls
  if [[ -f /var/run/rll-secret ]]; then
    mkdir -p /var/run/rightlink
    mv /var/run/rll-secret /var/run/rightlink/secret
  fi
fi

# Create or add to log file
echo "Logging to /var/log/rightlink.log" 1>&2
exec > >(tee -a /var/log/rightlink.log)
exec 2>&1
touch /var/log/rightlink.log && chmod 600 /var/log/rightlink.log

# Sleep command added on exit ensure output to stdout finishes before returning to command prompt
trap "sleep 1" EXIT

echo "`date` Starting Disablement"

# ===== Default configuration

# Determine bin and lib directory used on installation, /usr/local/bin|lib or /opt/bin|lib due to read-only of /usr
if [[ -e /usr/local/bin/rightlink ]]; then
  BIN_DIR=/usr/local/bin
  LIB_DIR=/usr/local/lib
else
  BIN_DIR=/opt/bin
  LIB_DIR=/opt/lib
fi

DEFAULT_SERVER="my.rightscale.com"
RSC=$BIN_DIR/rsc
RL=$BIN_DIR/rightlink

if [[ ! "$PATH" =~ $BIN_DIR ]]; then
  export PATH="$PATH:$BIN_DIR"
fi

show_help() {
  cat <<EOT
Usage: ${0##*/} [-options...]
Disable a RightScale server that had been a previously enabled server.
  -k refresh_token      RightScale API refresh token from the dash Settings>API Credentials (req.)
  -s server_href        this server's HREF (disables self-detection) (ex. /api/servers/123456789)
  -a api_hostname       the hostname for the RightScale API, default: $DEFAULT_SERVER
  -f                    override command prompt confirmation for disablement
  -h                    show this help

Required Inputs: -k refresh_token
EOT
}

# ===== Parse arguments

refresh_token=
cloud_type=
server_href=
api_hostname=
confirmation_override=

while getopts "hvfk:s:a:" opt; do
  case "$opt" in
    h) show_help; exit 0 ;;
    v) set -x ;;
    k) refresh_token="$OPTARG" ;;
    s) server_href="$OPTARG" ;;
    a) api_hostname="$OPTARG" ;;
    f) confirmation_override=true ;;
    '?') show_help >&2; exit 1 ;;
  esac
done

# Shift off the options and optional --.
shift "$((OPTIND-1))"

if [[ `id -u` -ne 0 ]]; then
  echo "ERROR: The script must be run with root privileges." >&2
  exit 1
fi

if [[ -z "$refresh_token" ]]; then
  echo "ERROR: -k refresh_token missing." >&2
  show_help >&2
  exit 1
fi

# ===== Download RSC command line tool if not present

if ! which rsc >/dev/null; then
  if ! which curl >/dev/null && ! which wget >/dev/null; then
    echo "ERROR: Cannot find wget or curl: one is required for this script." >&2
    exit 1
  fi

  RSC_URL="https://binaries.rightscale.com/rsbin/rsc/v6.2.0/rsc-linux-amd64.tgz"

  cd /tmp
  rm -rf rsc rsc-linux-amd64.tgz
  for s in 2 4 6 8 12 16 20 24 32 64; do
    if wget -q -O rsc-linux-amd64.tgz $RSC_URL 2>/dev/null || curl -s -o rsc-linux-amd64.tgz $RSC_URL ; then
      break
    fi
    echo "`date` Download error, sleeping $s seconds"
    sleep "$s"
  done
  # Expand the archive into /tmp/rsc
  tar zxf rsc-linux-amd64.tgz
  cd rsc

  # ===== Install RSC command line tool

  # Put the binary into a bin directory
  mkdir -p ${RSC%/*}
  mv rsc $RSC
  chmod +x $RSC
fi

if [[ -z "$api_hostname" ]]; then
  if [[ -f /var/lib/rightscale-identity ]]; then
    source /var/lib/rightscale-identity
    if [[ -n "$http_proxy" ]]; then
      export https_proxy="$http_proxy"
    fi
    # Translate old user-data style to updated style
    if [[ -n "$RS_api_hostname" ]]; then
      api_hostname="$RS_api_hostname"
    fi
  elif [[ -n "$DEFAULT_SERVER" ]]; then
    api_hostname="$DEFAULT_SERVER"
  else
    echo "ERROR: -a api_hostname missing." >&2
    show_help >&2
    exit 1
  fi
fi
echo "Using API Host $api_hostname"

# Check for Server HREF using user-data to make an api call
if [[ -z "$server_href" ]]; then
  if [[ -f /var/lib/rightscale-identity ]]; then
    if ps cax | grep rightlink >/dev/null; then
      echo -n "Locating Server HREF using user-data on file ... "
      server_href=$(rsc --rl10 --x1 ':has(.rel:val("parent")).href' cm15 \
                  index_instance_session /api/sessions/instance 2>/dev/null)
      if [[ -n "$server_href" ]]; then
        echo "server_href=$server_href"
      else
        echo "server_href not found."
      fi
    fi
  fi
fi

# Check for Server HREF in the log file created by enable script
if [[ -z "$server_href" ]]; then
  if [[ -f /var/log/rightlink.log ]]; then
    echo -n "Locating Server HREF from log files ... "
    log_file=$(cat /var/log/rightlink.log)
    re='((C|c)reated server: |Checking to see if instance is already (wrapped|enabled) \.\.\. )server_href=(/api/servers/[0-9]+)(.*)'
    while [[ "$log_file" =~ $re ]]; do
      server_href="${BASH_REMATCH[4]}"
      log_file="${BASH_REMATCH[5]}" # this is the rest of the log file
    done
    if [[ -n "$server_href" ]]; then
      echo "server_href=$server_href"
    else
      echo "server_href not found."
    fi
  fi
fi

# Take out the /deployments/123456789 from the HREF if it's there
server_href=$(echo "$server_href" | sed -r 's_/deployments/[0-9]+__')

# Check to make sure server_href exists and that it is valid
re='/api/servers/[0-9]+'
if [[ -z "$server_href" ]]; then
  echo "ERROR: -s server_href missing." >&2
  show_help >&2
  exit 1
elif [[ ! "$server_href" =~ $re ]]; then
  echo "ERROR: -s server_href malformed." >&2
  show_help >&2
  exit 1
fi

# Querying details about current server
echo -n "Finding current server ... "
json=$(rsc --key "$refresh_token" --host "$api_hostname" cm15 show "$server_href")
if [[ -z "$json" ]]; then
  echo "ERROR: Server not found." >&2
  exit 1
fi

# Make sure that the server is in a valid state to be disabled
state=$(echo "$json" | rsc --x1 '.state' json 2>/dev/null)
if [[ "$state" != "operational" && "$state" != "booting" && ! "$state" =~ stranded ]]; then
  echo "ERROR: Server found, but in the wrong state: $state." >&2
  exit 1
else
  echo "Server found and is in state: $state."
  # Store server name
  server_name=$(echo "$json" | rsc --x1 ':root > .name' json 2>/dev/null)
fi

echo ""
echo "The following will be deleted locally:"

# Locate RightLink
if [[ -f $RL ]]; then
  echo "RightLink           $RL"
fi
# Old Version of RightLink
if [[ -f $RLL ]]; then
  echo "RightLink           $RLL"
fi
# Locate RSC
if [[ -f $RSC ]]; then
  echo "RSC                 $RSC"
fi
if [[ -f /var/lib/rightscale-identity ]]; then
  echo "User-Data           /var/lib/rightscale-identity"
fi
if [[ -f /var/run/rightlink/secret ]]; then
  echo "Secret              /var/run/rightlink/secret"
fi
if [[ -d /var/lib/rightlink ]]; then
  echo "Data directory      /var/lib/rightlink"
fi
if [[ -d /var/lib/rightlink_keys ]]; then
  echo "Keys directory      /var/lib/rightlink_keys"
fi
# Check for old Secret file (even though it should have been moved)
if [[ -f /var/run/rll-secret ]]; then
  echo "Secret              /var/run/rll-secret"
fi
if [[ -f /var/run/rightlink/state ]]; then
  echo "State               /var/run/rightlink/state"
fi
# Check for old State file
if [[ -f /var/run/rightlinklite.state ]]; then
  echo "State               /var/run/rightlinklite.state"
fi
# Locate attachments, cookbooks, scripts, libraries, and folders
if [[ -d /var/spool/rightlink ]]; then
  echo "Attachments         /var/spool/rightlink"
fi
if [[ -d /var/spool/rll ]]; then
  echo "Attachments         /var/spool/rll"
fi
if [[ -f /etc/sudoers.d/90-rightscale ]]; then
  echo "RightLink sudo      /etc/sudoers.d/90-rightscale"
fi
if [[ -f /etc/sudoers.d/90-rightscale-sudo-users ]]; then
  echo "RightScale sudo     /etc/sudoers.d/90-rightscale-sudo-users"
fi
if [[ -f ${BIN_DIR}/rs-ssh-keys.sh ]]; then
  echo "SSH Key helper      ${BIN_DIR}/rs-ssh-keys.sh"
fi
if [[ -L ${LIB_DIR}/libnss_rightscale.so ]]; then
  echo "NSS Plugin          ${LIB_DIR}/libnss_rightscale.so*"
fi
if [[ -d /var/lib/rightlink ]]; then
  echo "rightlink lib dir   /var/lib/rightlink"
fi
if [[ -d /var/lib/rightlink_keys ]]; then
  echo "Keys directory      /var/lib/rightlink_keys"
fi

# Locate init scripts
if [[ -f /etc/init/rightlink.conf ]]; then
  echo "Init Script         /etc/init/rightlink.conf"
elif [[ -f /etc/init/rightlinklite.conf ]]; then
  echo "Init Script         /etc/init/rightlinklite.conf"
elif [[ -f /etc/systemd/system/rightlink.service ]]; then
  echo "Init Script         /etc/systemd/system/rightlink.service"
elif [[ -f /etc/systemd/systemd/rightlinklite.service ]]; then
  echo "Init Script         /etc/systemd/system/rightlinklite.service"
elif [[ -f /etc/init.d/rightlink ]]; then
  echo "Init Script         /etc/init.d/rightlink"
elif [[ -f /etc/init.d/rightlinklite ]]; then
  echo "Init Script         /etc/init.d/rightlinklite"
fi
# Locate temporary files
if [[ -f /tmp/rightlink.tgz ]]; then
  echo "RL10 Archive        /tmp/rightlink.tgz"
elif [[ -f /tmp/rll.tgz ]]; then
  echo "RL10 Archive        /tmp/rll.tgz"
fi
if [[ -d /tmp/rightlink ]]; then
  echo "RL10 Scripts        /tmp/rightlink"
elif [[ -d /tmp/rll ]]; then
  echo "RL10 Scripts        /tmp/rll"
fi
if [[ -e /usr/local/share/rightnetwork/uninstall.sh ]]; then
  echo "Networking Scripts  /usr/local/share/rightnetwork"
fi

# Locate config files
echo ""
echo "The following configurations will be updated:"
# 'rightlink' user
if id -u "rightlink" >/dev/null 2>&1; then
  echo "User 'rightlink' removed"
fi
# nsswitch.con entry
if cut --delimiter=# --fields=1 /etc/nsswitch.conf | grep --quiet rightscale; then
  echo "Remove 'rightscale' from /etc/nsswitch.conf"
fi
# sshd_config entry
ssh_config_entry="AuthorizedKeysCommand ${BIN_DIR}/rs-ssh-keys.sh"
if cut --delimiter=# --fields=1 /etc/ssh/sshd_config | grep --quiet "${ssh_config_entry}"; then
  echo "Remove ${ssh_config_entry} from /etc/ssh/sshd_config"
fi
# PAM config entry
if cut --delimiter=# --fields=1 /etc/pam.d/sshd | grep --quiet "Added by RightScale Managed Login script"; then
	echo "Remove pam_mkhomedir.so entry from /etc/pam.d/sshd"
fi

# Prompt user for confirmation
echo ""
echo "The following Server will be RightScale disabled:"
echo "Server Name         ${server_name}"
echo "Server HREF         ${server_href}"

if [[ "$confirmation_override" == true ]]; then
  response="y"
else
  echo ""
  read -r -p "Do you want to proceed? [y/N] " response </dev/tty
fi

echo ""

if [[ "$response" =~ ^([yY][eE][sS]|[yY])$ ]]; then
  # Get cloud href from current instance href
  current_instance_href=$(rsc --x1 ':has(.rel:val("current_instance")).href' json <<<"$json")
  cloud_href="${current_instance_href%/instances/*}"
  # Get cloud type
  cloud_type=$(rsc --key "$refresh_token" --host "$api_hostname" --x1 '.cloud_type' cm15 show "$cloud_href")
  if [[ $cloud_type != uca ]]; then
    # Execute disable call
    instance_href=$(rsc --key "$refresh_token" --host "$api_hostname" --xh 'location' cm15 unwrap "$server_href" 2>&1)
  fi
  # Confirm disable success / failure
  re='^/api/clouds/[0-9]+/instances/.*$'
  if [[ $cloud_type != uca ]] && [[ ! "$instance_href" =~ $re ]]; then
    # If server failed to disable, output response from API
    echo "ERROR: Failed to disable server: ${server_href}"
    echo "$instance_href" >&2
    exit 1
  else
    # Delete RightLink
    rm -vf $RL
    # Delete Old RightLink
    rm -vf $RLL
    # Delete RightScale user-data
    rm -vf /var/lib/rightscale-identity
    # Delete RightScale data directory
    rm -vfr /var/lib/rightlink
    # Delete RightScale keys directory
    rm -vfr /var/lib/rightlink_keys
    # Delete Secret and State files
    rm -rvf /var/run/rightlink
    # Delete old State and Secret files (even though it should have been moved)
    rm -vf /var/run/rightlinklite.state
    rm -vf /var/run/rll-secret
    # Delete Attachments, Cookbooks and Scripts
    rm -rvf /var/spool/rightlink
    rm -rvf /var/spool/rll
    # Delete Temporary files
    rm -vf /tmp/rightlink.tgz
    rm -rvf /tmp/rightlink
    rm -vf /tmp/rll.tgz
    rm -rvf /tmp/rll
    rm -vf /tmp/rsc-linux-amd64.tgz
    rm -rvf /tmp/rsc
    # Delete sudoers file
    rm -vf /etc/sudoers.d/90-rightscale
    rm -vf /etc/sudoers.d/90-rightscale-sudo-users
    # Delete init scripts
    if [[ -e /etc/init/rightlink.conf ]] || [[ -e /etc/init/rightlinklite.conf ]]; then
      rm -vf /etc/init/rightlink.conf
      rm -vf /etc/init/rightlinklite.conf
    elif [[ -e /etc/systemd/system/rightlink.service ]] || [[ -e /etc/systemd/system/rightlinklite.service ]]; then
      rm -vf /etc/systemd/system/rightlink.service
      systemctl disable rightlink 2>/dev/null
      rm -vf /etc/systemd/system/rightlinklite.service
      systemctl disable rightlinklite 2>/dev/null
    elif [[ -x /usr/sbin/update-rc.d ]] || [[ -x /sbin/chkconfig ]]; then
      rm -vf /etc/init.d/rightlink
      rm -vf /etc/init.d/rightlinklite
      if [[ -f /etc/init.d/cloud-init-user-scripts ]]; then
        sed -i 's/always/once-per-instance/' /etc/init.d/cloud-init-user-scripts
      fi
      if [[ -f /etc/init.d/cloud-final ]]; then
        sed -i '/Required-Start: *cloud-config/s/cloud-config/$all cloud-config/' /etc/init.d/cloud-final
      fi
      if [[ -x /usr/sbin/update-rc.d ]]; then
        /usr/sbin/update-rc.d rightlink remove 2>/dev/null
        /usr/sbin/update-rc.d rightlinklite remove 2>/dev/null
      elif [[ -x /sbin/chkconfig ]]; then
        /sbin/chkconfig --del rightlink 2>/dev/null
        /sbin/chkconfig --del rightlinklite 2>/dev/null
      fi
    fi
    if [[ -w /etc/cloud/cloud.cfg ]]; then
      sed -i -e '/ - \[scripts-user, always\]/s/\[scripts-user, always\]/scripts-user/' /etc/cloud/cloud.cfg
    fi
    # Remove rightscale_network components
    if [[ -e /usr/local/share/rightnetwork/uninstall.sh ]]; then
      echo "Removing RightScale networking scripts"
      chmod a+x /usr/local/share/rightnetwork/uninstall.sh
      /usr/local/share/rightnetwork/uninstall.sh
    fi
    # Stop RightLink without running decommission bundle
    pkill -9 rightlink$
    pkill -9 rightlinklite$
    if [[ $cloud_type == uca ]]; then
      rsc --key "$refresh_token" --host "$api_hostname" cm15 terminate "$server_href"
      until [[ $(rsc --key "$refresh_token" --host "$api_hostname" --x1 '.state' cm15 show "$server_href") == inactive ]]; do
        sleep 1
      done
      rsc --key "$refresh_token" --host "$api_hostname" cm15 destroy "$server_href"
    fi
    # Delete RSC
    rm -vf $RSC
    # Check to see if the old version of the RightLink upstart log exists
    if [[ -f /var/log/upstart/rightlinklite.log ]]; then
      # Wait to make sure that upstart knows that RightLink has stopped before renaming the log
      while [[ `initctl status rightlinklite 2>/dev/null` =~ running ]]; do
        sleep 1
      done
      mv -v /var/log/upstart/rightlinklite.log /var/log/upstart/rightlink.log
    fi
    # Delete 'rightlink' user
    if id -u "rightlink" >/dev/null 2>&1; then
      userdel -rf rightlink 2>/dev/null || true
      groupdel rightlink 2>/dev/null || true
    fi
    # Remove rightscale NSS plugin from /etc/nsswitch.conf
    sed -i '/^\(passwd\|group\|shadow\)/ s/\s\?rightscale\s*/ /; s/\s*$//' /etc/nsswitch.conf
    # Remove AuthorizedKeysCommand and AuthorizedKeysCommandUser from sshd_config
    sed -i '/^AuthorizedKeysCommand \/usr\/local\/bin\/rs-ssh-keys.sh$/d' /etc/ssh/sshd_config
    sed -i '/^AuthorizedKeysCommandUser nobody$/d' /etc/ssh/sshd_config
    # Remove rs-ssh-keys.sh
    rm -vf $BIN_DIR/rs-ssh-keys.sh
    # Remove libnss_rightscale NSS plugin
    rm -vf $LIB_DIR/libnss_rightscale.*
    # Remove pam_mkhomedir line from /etc/pam.d/sshd
    sed  -i '/^# Added by RightScale Managed Login script$/ {N; /^#.*session required pam_mkhomedir.so skel=\/etc\/skel\/ umask=0022$/d}' /etc/pam.d/sshd
    # Remove /var/lib/rightlink folder
    rm -frv /var/lib/rightlink/
    # Remove /var/lib/rightlink_keys folder
    rm -frv /var/lib/rightlink_keys/

    if [[ $cloud_type != uca ]]; then
      echo ""
      echo "The disabled instance is located at:"
      echo "Instance HREF       ${instance_href}"
    fi
  fi
else
  # Put the log files and secret file back to their original state if the server was an 10.0.rc version
  if [[ $($RLL --version 2>/dev/null) =~ (10.0.rc0|10.0.rc1|10.0.rc2) ]]; then
    mv /var/log/rightlink.log /var/log/rightlinklite.log
    mv /var/run/rightlink/secret /var/run/rll-secret
    rm -f /var/run/rightlink
  fi
  echo "Exiting script with no changes."
  exit 0
fi

echo ""
echo "`date` Disablement complete.  Server has been reverted to pre-enablement state."
