#! /bin/bash -e

# This script will install/upgrade RightLink. This script is meant to be used to create
# custom images with RightLink pre-installed, such as when creating private cloud
# images or snapshotting existing images. If RightLink already exists on the instance,
# this script will replace it with version 10.6.4.
#
# RightLink depends on sudo and cloud-init. and this script depends on curl and tar.
# On some clouds, a custom cloud-init may be required. For details, see
# http://docs.rightscale.com/rl10/reference/rl10_cloud_init_installation.html
#
# For detailed instruction, see http://docs.rightscale.com/rl10/reference/rl10_install.html

# Create a log file
echo "Logging to /var/log/rightlink.log" 1>&2
exec > >(tee -a /var/log/rightlink.log)
exec 2>&1
# Sleep command added on exit ensure output to stdout finishes before returning to command prompt
trap "sleep 1" EXIT
touch /var/log/rightlink.log && chmod 600 /var/log/rightlink.log

# Determine if /usr/local/bin is read-only. If so, use /opt/bin to install executables
mkdir -p /usr/local/bin || true
[ -w /usr/local/bin ] && BIN_DIR=/usr/local/bin || BIN_DIR=/opt/bin

# Where to download the install package from, note that 10.6.4 gets replaced by the branch name
# in the CI build process
URL="https://rightlink.rightscale.com/rll/10.6.4/rightlink.tgz"
RL=$BIN_DIR/rightlink
RSC=$BIN_DIR/rsc

if [[ ! "$PATH" =~ $BIN_DIR ]]; then
  export PATH="$PATH:$BIN_DIR"
fi

if ! which curl >/dev/null && ! which wget >/dev/null; then
  echo "ERROR: Cannot find curl or wget: one is required for this script." >&2
  exit 1
fi

if ! which tar >/dev/null; then
  echo "ERROR: Cannot find tar: it is required for this script." >&2
  exit 1
fi

if ! which sudo >/dev/null; then
  echo "ERROR: Cannot find sudo: it is required for RightLink." >&2
  exit 1
fi

# Cloud-init is required for getting userdata for RightLink boot.
if ! which cloud-init >/dev/null && ! which coreos-cloudinit >/dev/null; then
  echo "ERROR: Cannot find cloud-init: it is required for RightLink startup." >&2
  echo "For certain clouds, a custom cloud-init may be required." >&2
  echo "For details, see http://docs.rightscale.com/rl10/reference/rl10_cloud_init_installation.html" >&2
  exit 1
fi

if dpkg --get-selections 2>/dev/null | grep -E "^(rightlink6|rightlink5|rightscale)\s" >/dev/null || \
  rpm -qa --queryformat '%{Name}\n' 2>/dev/null | grep -E "^(rightlink6|rightlink5|rightscale)$" >/dev/null; then
  echo "ERROR: Prior incompatible RightLink version found." >&2
  echo "RightLink 6 or any previous version must be uninstalled before proceeding." >&2
  echo "Please visit https://support.rightscale.com/12-Guides/RightLink_6/Uninstall_RightLink for uninstall instructions." >&2
  exit 1
fi

show_help() {
  cat <<EOT
Usage: ${0##*/} [-options...]
Installs/Upgrades RightLink on an an image. RightLink depends on sudo and cloud-init.
For Softlayer, a custom cloud-init is required. For details, see
http://docs.rightscale.com/rl10/reference/rl10_cloud_init_installation.html
  -n   skip installation of RightScale network configuration scripts for RCA-V
  -h   show this help
  -v   verbose mode
EOT
}

networking_scripts_arg=""

while getopts "hnv" opt; do
  case "$opt" in
    h) show_help; exit 0 ;;
    n) networking_scripts_arg="-n"; ;;
    v) set -x ;;
    '?') show_help >&2; exit 1 ;;
  esac
done

echo "`date` Starting RightLink 10.6.4 install."

# ===== Download and Expand RightLink
echo "Downloading tarball from $URL, extracting into /tmp/rightlink"
cd /tmp
rm -rf rightlink rightlink.tgz
for s in 2 4 6 8 12 16 20 24 32 64; do
  if wget -q -O rightlink.tgz $URL 2>/dev/null || curl -s -o rightlink.tgz $URL ; then
    break
  fi
  echo "`date` Download error, sleeping $s seconds"
  sleep $s
done
# Expand the archive into /tmp/rightlink
tar zxf rightlink.tgz
cd rightlink

# ===== Install RightLink
chmod +x ./install.sh
./install.sh $networking_scripts_arg

# clean up
rm -fr /tmp/rightlink /tmp/rightlink.tgz
