#! /bin/bash

# This script uninstalls a previous RightLink from disk. If the server has been
# "enabled" into a RightScale server, it won't disable it and turn it back into
# an instance. For that, see rightlink.disable.sh.

# Create or add to log file
echo "Logging to /var/log/rightlink.log" 1>&2
exec > >(tee -a /var/log/rightlink.log)
exec 2>&1
touch /var/log/rightlink.log && chmod 600 /var/log/rightlink.log

# Sleep command added on exit ensure output to stdout finishes before returning to command prompt
trap "sleep 1" EXIT

echo "`date` Starting Uninstall"

# Determine if /usr/local/bin is read-only. If so, use /opt/bin to install executables
mkdir -p /usr/local/bin || true
[ -w /usr/local/bin ] && BIN_DIR=/usr/local/bin || BIN_DIR=/opt/bin

# ===== Default configuration
RSC=$BIN_DIR/rsc
RL=$BIN_DIR/rightlink

if [[ ! "$PATH" =~ $BIN_DIR ]]; then
  export PATH="$PATH:$BIN_DIR"
fi

show_help() {
  cat <<EOT
Usage: ${0##*/} [-options...]
Uninstalls RightLink.
  -f                    override command prompt confirmation for uninstall
  -v                    verbose mode
  -h                    show this help
EOT
}

# ===== Parse arguments
while getopts "hvf" opt; do
  case "$opt" in
    h) show_help; exit 0 ;;
    v) set -x ;;
    f) confirmation_override=true ;;
    '?') show_help >&2; exit 1 ;;
  esac
done

# Shift off the options and optional --.
shift "$((OPTIND-1))"

if [[ `id -u` -ne 0 ]]; then
  echo "ERROR: The script must be run with root privileges." >&2
  exit 1
fi

echo ""
echo "The following will be deleted locally:"

# Locate RightLink
if [[ -f $RL ]]; then
  echo "RightLink           $RL"
fi
# Locate RSC
if [[ -f $RSC ]]; then
  echo "RSC                 $RSC"
fi
if [[ -f /var/lib/rightscale-identity ]]; then
  echo "User-Data           /var/lib/rightscale-identity"
fi
if [[ -f /var/run/rightlink/secret ]]; then
  echo "Secret              /var/run/rightlink/secret"
fi
if [[ -f /var/run/rightlink/state ]]; then
  echo "State               /var/run/rightlink/state"
fi
# Locate Attachments, Cookbooks and Scripts
if [[ -d /var/spool/rightlink ]]; then
  echo "Attachments         /var/spool/rightlink"
fi
# Locate 'rightscale' user
if id -u "rightscale" >/dev/null 2>&1; then
  echo "User rightscale     /home/rightscale"
fi
# Locate 'rightlink' user
if id -u "rightlink" >/dev/null 2>&1; then
  echo "User rightlink"
fi
if [[ -f /etc/sudoers.d/90-rightscale ]]; then
  echo "Sudoers             /etc/sudoers.d/90-rightscale"
fi
# Locate init scripts
if [[ -f /etc/init/rightlink.conf ]]; then
  echo "Init Script         /etc/init/rightlink.conf"
elif [[ -f /etc/systemd/system/rightlink.service ]]; then
  echo "Init Script         /etc/systemd/system/rightlink.service"
elif [[ -f /etc/init.d/rightlink ]]; then
  echo "Init Script         /etc/init.d/rightlink"
fi
# Locate temporary files
if [[ -f /tmp/rightlink.tgz ]]; then
  echo "RL10 Archive        /tmp/rightlink.tgz"
fi
if [[ -d /tmp/rightlink ]]; then
  echo "RL10 Scripts        /tmp/rightlink"
fi
if [[ -e /usr/local/share/rightnetwork/uninstall.sh ]]; then
  echo "Networking scripts  /usr/local/share/rightnetwork"
fi

if [[ "$confirmation_override" == true ]]; then
  response="y"
else
  echo ""
  read -r -p "Do you want to proceed? [y/N] " response </dev/tty
fi

echo ""

if [[ "$response" =~ ^([yY][eE][sS]|[yY])$ ]]; then
  # Delete RightLink
  rm -vf $RL
  # Delete RSC
  rm -vf $RSC
  # Delete RightScale user-data
  rm -vf /var/lib/rightscale-identity
  # Delete Secret and State files
  rm -rvf /var/run/rightlink
  # Delete Attachments, Cookbooks and Scripts
  rm -rvf /var/spool/rightlink
  # Delete Temporary files
  rm -vf /tmp/rightlink.tgz
  rm -rvf /tmp/rightlink
  rm -vf /etc/sudoers.d/90-rightscale
  # Delete init scripts
  if [[ -e /etc/init/rightlink.conf ]]; then
    rm -vf /etc/init/rightlink.conf
  elif [[ -e /etc/systemd/system/rightlink.service ]]; then
    rm -vf /etc/systemd/system/rightlink.service
    systemctl disable rightlink
  elif [[ -x /usr/sbin/update-rc.d ]] || [[ -x /sbin/chkconfig ]]; then
    rm -vf /etc/init.d/rightlink
    if [[ -f /etc/init.d/cloud-init-user-scripts ]]; then
      sed -i 's/always/once-per-instance/' /etc/init.d/cloud-init-user-scripts
    fi
    if [[ -f /etc/init.d/cloud-final ]]; then
      sed -i '/Required-Start: *cloud-config/s/cloud-config/$all cloud-config/' /etc/init.d/cloud-final
    fi
    if [[ -x /usr/sbin/update-rc.d ]]; then
      /usr/sbin/update-rc.d rightlink remove
    elif [[ -x /sbin/chkconfig ]]; then
      /sbin/chkconfig --del rightlink
    fi
  fi
  if [[ -w /etc/cloud/cloud.cfg ]]; then
    sed -i -e '/ - \[scripts-user, always\]/s/\[scripts-user, always\]/scripts-user/' /etc/cloud/cloud.cfg
  fi
  # Remove rightscale_network components
  if [[ -e /usr/local/share/rightnetwork/uninstall.sh ]]; then
    echo "Removing RightScale networking scripts"
    chmod a+x /usr/local/share/rightnetwork/uninstall.sh
    /usr/local/share/rightnetwork/uninstall.sh
  fi
  # Stop RightLink without running decommission bundle
  pkill -9 rightlink$
  # Delete 'rightscale' user
  if id -u "rightscale" >/dev/null 2>&1; then
    userdel -rf rightscale 2>/dev/null || true
    groupdel rightscale 2>/dev/null || true
  fi
  # Delete 'rightlink' user
  if id -u "rightlink" >/dev/null 2>&1; then
    userdel -rf rightlink 2>/dev/null || true
    groupdel rightlink 2>/dev/null || true
  fi
else
  echo "Exiting script with no changes."
  exit 0
fi

echo ""
echo "`date` Uninstallation complete."
