#!/bin/bash

# USER DEFINED VARIABLES
NETWORK_SCRIPTS_DIR=${NETWORK_SCRIPTS_DIR:=/etc/sysconfig/network-scripts}
NETWORK_INTERFACES_DIR=${NETWORK_INTERFACES_DIR:=/etc/network/interfaces.d}
NETWORK_INTERFACES_FILE=${NETWORK_INTERFACES_FILE:=/etc/network/interfaces}
VMTOOLSD_NAME=${VMTOOLSD_NAME:=vmtoolsd}
SYS_NET_DIR=${SYS_NET_DIR:=/sys/class/net}
CMD_ROUTE_SHOW=${CMD_ROUTE_SHOW:="ip route show"}
CMD_ROUTE_ADD=${CMD_ROUTE_ADD:="ip route add"}
CMD_ROUTE_DEFAULT_GATEWAY=${CMD_ROUTE_DEFAULT_GATEWAY:="ip -4 route list 0/0"}
CMD_ROUTE_ADD_DEFAULT_GATEWAY=${CMD_ROUTE_ADD_DEFAULT_GATEWAY:="route add default gw"}
CMD_ROUTE_DELETE_DEFAULT_GATEWAY=${CMD_ROUTE_DELETE_DEFAULT_GATEWAY:="route del default gw"}
CMD_IP_SHOW=${CMD_IP_SHOW:="ip addr show"}
DEBUG=${DEBUG:=false}

# CONSTANTS
declare -r FLAG_ON='true'

# ERRORS
declare -r E_DID_NOTHING=1
declare -r E_UNSUPPORTED_DISTRO=2
declare -r E_NOT_DEFINED_NETMASK=10
declare -r E_METADATA_NOT_READY=11

declare -r E_INVALID_DEVICE_NAME=101
declare -r E_INVALID_IPV4_ADDRESS=102
declare -r E_INVALID_NETMASK=103
declare -r E_INVALID_GATEWAY=104
declare -r E_INVALID_CIDR=105
declare -r E_INVALID_NAT_SERVER_IPV4=106
declare -r E_NOT_FOUND_ROUTE_DEVICE=107


function usage
{
  echo "Usage example:"
  echo "$0 (--init|--routes)[ --debug|-d]"
}

function output # message
{
  echo $1
}

# Display error and exit
# if code is not provided,
# then 1 used by default
function error # message, code
{
  echo "ERROR: $1";
  if ! [ -z "${2}" ]; then exit $2 >&2; fi
  exit ${E_DID_NOTHING}
}

# Output message, if debug mode is on
function debug # message
{
  if [[ "${DEBUG}" == "${FLAG_ON}" ]]; then
    echo "DEBUG: ${1}"
  fi
}

function name_from # pair
{
  echo "$(echo ${1%=*} | awk '{print toupper($0)}')"
}

function value_from # pair
{
  echo "${1#*=}"
}

function nat_server_ip_from # route_str
{
  echo "${1%,*}"
}

function cidr_from # route_str
{
  echo "${1#*,}"
}

function cidr_to_mask ()
{
   # Number of args to shift, 255..255, first non-255 byte, zeroes
   set -- $(( 5 - ($1 / 8) )) 255 255 255 255 $(( (255 << (8 - ($1 % 8))) & 255 )) 0 0 0
   [ $1 -gt 1 ] && shift $1 || shift
   echo ${1-0}.${2-0}.${3-0}.${4-0}
}

# Wait for metadata to be ready. A couple use cases to worry about:
#   1. Existing instance was enabled. There will be no metadata and we want to quit gracefully in this case.
#   2. Server launched via RCA-V. RCA-V needs time to get the instance_id and other values, so those values will be
#      set to discovering. We want to wait those values to be set in a loop. vmtools may return a non-zero exit status
#      for the initial poll or two at boot also while VMware tools initializes, so catch that as well.
function wait_for_metadata
{
  # Up 10 minutes total wait
  local max_retries=60
  local retry_delay=10
  local retry=0
  local done_waiting=1
  local data
  local status

  echo "Attempting to read metadata"
  while [ $retry -lt $max_retries ]; do
    data=$($VMTOOLSD_BIN "--cmd=info-get guestinfo.metadata" 2>&1)
    status=$?
    if [[ "$data" =~ "No value found" ]]; then
      # Note this check comes before the status check -- a status of 1 will be returned but that's ok in this case.
      debug "Done waiting continue"
      done_waiting=0
      break
    elif [[ "$status" -ne 0 ]]; then
      # Can happen for first poll or two at bootup
      debug "Non zero exit status from vmtoolsd command"
      let retry=retry+1
      sleep $retry_delay
    elif [[ $data =~ "=discovering" ]]; then
      debug "Data is not ready, sleeping"
      let retry=retry+1
      sleep $retry_delay
    else
      debug "Done waiting continue"
      done_waiting=0
      break
    fi
  done
  if ! [ $done_waiting == 0 ]; then
    error "Metadata is not ready" ${E_METADATA_NOT_READY}
  fi
  if [[ "$data" =~ "No value found" ]]; then
    echo "No metadata found, not configuring metadata"
    exit 0
  fi
  IFS='&'
  for pair in $data; do
    export "$(name_from ${pair})=$(value_from ${pair})"
  done
  unset IFS
  echo "Done waiting for metadata"
}

function _rs_variable_name # numeric, suffix
{
  echo "RS_IP${1}_${2}"
}

function addr_value # numeric
{
  name=$(_rs_variable_name $1 "ADDR")
  echo ${!name}
}

function netmask_value # numeric
{
  name=$(_rs_variable_name $1 "NETMASK")
  echo ${!name}
}

function gateway_value # numeric
{
  name=$(_rs_variable_name $1 "GATEWAY")
  echo ${!name}
}

function mac_value # numeric
{
  name=$(_rs_variable_name $1 "MAC")
  echo ${!name}
}

function nameservers_value # numeric
{
  name=$(_rs_variable_name $1 "NAMESERVERS")
  echo ${!name}
}

function route_value # numeric
{
  name="RS_ROUTE${1}"
  echo ${!name}
}

function device_name_from_mac # mac
{
  name=$(grep -i -l "$1" $SYS_NET_DIR/*/address | sed "s#$SYS_NET_DIR/##" | sed "s#/address##")
  echo $name
}

function route_device # nat_server_ip, is_init
{
  local device
  local ipaddr
  local netmask
  local num
  for num in $(static_ip_numerals); do
    ipaddr=$(addr_value "$num")
    netmask=$(netmask_value "$num")
    if in_adaptor_range "$ipaddr" "$netmask" $1; then
      device="$(device_name_from_mac "$(mac_value $num)")"; break
    fi
  done
  if [[ "$is_init" == "0" ]] && [[ -z "$device" ]]; then
    devices=$($CMD_IP_SHOW | grep '^[0-9]' | grep -v 'lo:' | cut -d" " -f 2 | sed 's/://')
    for dev in $devices; do
      mac=$(from_device_config $dev 'link/ether')
      ip_and_mask=$(from_device_config $dev 'inet')
      if [[ -n "$mac" ]] && [[ -n "$ip_and_mask" ]]; then
        ipaddr=$(echo "$ip_and_mask" | cut -d'/' -f1)
        netmask_num=$(echo "$ip_and_mask" | cut -d'/' -f2)
        netmask=$(cidr_to_mask $netmask_num)
        if in_adaptor_range "$ipaddr" "$netmask" $1; then
          device="$dev"; break
        fi
      fi
    done
  fi
  echo "$device"
}

function seen_route_str # cidr, nat_server_ip
{
  echo "${1}=${2}||"
}

function seen_route # cidr, nat_server_ip, seen_routes
{
  if echo $3 | grep "$(seen_route_str "$1" "$2")"; then
    echo "$3"
    return 0
  else
    echo "${3}$(seen_route_str "$1" "$2")"
    return 1
  fi
}

# Verifies the format of an IPv4 address
function validate_ip4 # ipv4_address
{
  if ! [[ $1 =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
    return 1
  fi
  return 0
}

# Verifies the format of an IPv4 cider address
function valid_ipv4_cidr # cidr
{
  if ! [[ $1 =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/[0-9]{1,2}$ ]]; then
    return 1
  fi
  return 0
}

function network_route_exists # cidr, nat_server_ip
{
  network="$1"
  if ! [ "$1" == "default" ]; then
    if [ "$(echo $1 | cut -d"/" -f2 )" == "32" ]; then # Single range CIDR
      network="$(echo $1 | cut -d"/" -f1 )"
    fi
  fi
  $CMD_ROUTE_SHOW | grep -e ${network}.*via.*${2}
  return $?
}

function separate_configs_enabled
{
  $(cat ${NETWORK_INTERFACES_FILE} | grep -e "/source \/etc\/network\/interface.d/")
  return $?
}

function enable_separate_configs
{
  cat > ${NETWORK_INTERFACES_FILE} << EOF
# File managed by RightScale
# DO NOT EDIT
auto lo
iface lo inet loopback

source /etc/network/interfaces.d/*.cfg
EOF
}

function init_device_config_file # device, file
{
  echo $(config_data_dhcp ${1}) > ${2}
}

function config_file # device
{
  ${CURRENT_DISTRO}_config_file "$@"
}

function ubuntu_config_file #device
{
  if ! separate_configs_enabled; then
    enable_separate_configs
  fi
  mkdir -p "${NETWORK_INTERFACES_DIR}"
  echo "${NETWORK_INTERFACES_DIR}/${1}.cfg"
}

function centos_config_file # device
{
  mkdir -p "${NETWORK_SCRIPTS_DIR}"
  echo "${NETWORK_SCRIPTS_DIR}/ifcfg-${1}"
}

function route_file # device
{
  ${CURRENT_DISTRO}_route_file "$@"
}

function ubuntu_route_file # device
{
  file=$(config_file "$1")
  if ! [ -e ${file} ]; then
    init_device_config_file ${1} ${file}
  fi
  echo ${file}
}

function centos_route_file # device
{
  mkdir -p "${NETWORK_SCRIPTS_DIR}"
  echo "${NETWORK_SCRIPTS_DIR}/route-${1}"
}

function write_adaptor_config # device, config_data, skip_if_exists
{
  local file=$(config_file $1)
  local skip_if_exists=${3:-false}
  if [[ -f $file && "$skip_if_exists" == "$FLAG_ON" ]]; then
    return 0
  fi
  echo "$2" > "$file"
}

function write_route_config # device, config_data
{
  local file=$(route_file "${1}")
  if ! grep "$2" $file >/dev/null 2>&1; then
    echo "$2" >> $file
  fi
}


function config_data_static # device, is_primary, ipaddr, netmask, gateway, nameservers
{
  ${CURRENT_DISTRO}_config_data_static "$@"
}


function ubuntu_config_data_static # device, is_primary, ipaddr, netmask, gateway, nameservers
{
  local device="$1"
  local is_primary="$2"
  local ipaddr="$3"
  local netmask="$4"
  local gateway="$5"
  local nameservers="$6"
  cat << EOF
# File managed by RightScale
# DO NOT EDIT
auto ${device}
iface ${device} inet static
address ${ipaddr}
netmask ${netmask}
EOF

  if [ -n "${gateway}" ]; then
    echo "gateway ${gateway}"
  fi
  if [ -n "${nameservers}" ]; then
    echo -n "dns-nameservers"
    IFS=','
    for nameserver in $(echo "$nameservers"); do
      echo -n " ${nameserver}"
    done
    echo ""
  fi
}

function centos_config_data_static # device, is_primary, ipaddr, netmask, gateway, nameservers
{
  local device="$1"
  local is_primary="$2"
  local ipaddr="$3"
  local netmask="$4"
  local gateway="$5"
  local nameservers="$6"
  local peerdns=no
  if [[ "$is_primary" == "1" ]]; then
    peerdns=yes
  fi
  cat << EOF
# File managed by RightScale
# DO NOT EDIT
DEVICE=${device}
BOOTPROTO=none
ONBOOT=yes
GATEWAY=${gateway}
NETMASK=${netmask}
IPADDR=${ipaddr}
USERCTL=no
PEERDNS=${peerdns}
NM_CONTROLLED=no
EOF
  if [ -n "$nameservers" ]; then
    IFS=','
    local index=1
    for nameserver in $(echo "$nameservers"); do
      echo "DNS${index}=${nameserver}"
      let index=index+1
    done
  fi
}

function config_data_dhcp # device
{
  ${CURRENT_DISTRO}_config_data_dhcp "$@"
}

function ubuntu_config_data_dhcp # device
{
cat << EOF
# File managed by RightScale
# DO NOT EDIT
auto ${1}
iface ${1} inet dhcp
EOF
}

function centos_config_data_dhcp # device, is_primary
{
  local peerdns=no
  if [[ "$is_primary" == "1" ]]; then
    peerdns=yes
  fi
cat << EOF
# File managed by RightScale
# DO NOT EDIT
DEVICE=${1}
BOOTPROTO=dhcp
ONBOOT=yes
PEERDNS=${peerdns}
EOF
}

function config_data_route # cidr, nat_server_ip
{
  ${CURRENT_DISTRO}_config_data_route "$@"
}

function ubuntu_config_data_route # cidr, nat_server_ip
{
  echo "up ip route add ${1} via ${2}"
}

function centos_config_data_route # cidr, nat_server_ip
{
  echo "${1} via ${2}"
}


function configure_network_adaptor # assignment, is_primary, device, ipaddr, netmask, gateway, nameservers
{
  local assignment="$1"
  local device="$2"
  local is_primary="$3"
  local ipaddr="$4"
  local netmask="$5"
  local gateway="$6"
  local nameservers="$7"

  if [[ "$assignment" == "dhcp" ]]; then
    skip_if_exists=$FLAG_ON
  else
    skip_if_exists=""
    if ! validate_ip4 $ipaddr; then 
      echo "Invalid IP address: '${ipaddr}'"; return ${E_INVALID_IPV4_ADDRESS}
    fi

    if ! validate_ip4 $netmask; then
      echo "Invalid netmask: '${netmask}'"; return ${E_INVALID_NETMASK}
    fi

    if [[ -n "$gateway" ]]; then 
      if ! validate_ip4 $gateway; then
        echo "Invalid gateway: '${gateway}'"; return ${E_INVALID_GATEWAY}
      fi
    fi
  fi

  config_data="$(config_data_$assignment "$device" "$is_primary" "$ipaddr" "$netmask" "$gateway" "$nameservers")"

  write_adaptor_config "$device" "$config_data" "$skip_if_exists"
}


# ARG0: decimal value
# RESULT: ip address string in form nnn.nnn.nnn.nnn
function dec2ip () {
    local ip dec=$@
    for e in {3..0}
    do
        ((octet = dec / (256 ** e) ))
        ((dec -= octet * 256 ** e))
        ip+=$delim$octet
        delim=.
    done
    printf '%s\n' "$ip"
}

# ARG0: ip address string in form nnn.nnn.nnn.nnn
# RESULT: decimal value
function ip2dec () {
    local a b c d ip=$@
    IFS=. read -r a b c d <<< "$ip"
    printf '%d\n' "$((a * 256 ** 3 + b * 256 ** 2 + c * 256 + d))"
}

# ARG0: integer in range 0..32
# RESULT: ip of mask. I.e. 24 -> 255.255.255.0
function mask_length2ip () {
  let full_mask=4294967295 # 2**32 - 1
  let mask_len=$((32-$1))
  let mask=$(($full_mask >> $mask_len))
  let mask=$(($mask << $mask_len))
  printf "$(dec2ip $mask)"
}

# ARG0: IP address in form nnn.nnn.nnn.nnn
# ARG1: Netmask in form nnn.nnn.nnn.nnn
# ARG2: Second ip address, checks if it falls in range specified by ARGV0, ARGV1 pair
# Example usage in_adaptor_range $RS_IP1_ADDR $RS_IP1_NETMASK $RS_ROUTE0
# RESULT: 0 if true, 1 if not
function in_adaptor_range() {
  let full_mask=4294967295 # 2**32 - 1
  let ip_addr_n=$(ip2dec $1)
  let ip_mask_n=$(ip2dec $2)
  route_ip="$(echo $3 | sed 's#,.*$##')"
  let route_n=$(ip2dec $route_ip)
  let lower_mask_length=$(($full_mask^$ip_mask_n))

  let lower_bound_n=$(($ip_addr_n & $ip_mask_n))
  let upper_bound_n=$(($lower_bound_n + $lower_mask_length))

  upper_bound_ip=$(dec2ip $upper_bound_n)
  lower_bound_ip=$(dec2ip $lower_bound_n)
  # echo -n route $route_ip ub_ip $upper_bound_ip lb_ip $lower_bound_ip in?
  if [[ $route_n -ge $lower_bound_n ]] && [[ $route_n -le $upper_bound_n ]]; then
    # echo " Y"
    return 0
  else
    # echo " N"
    return 1
  fi
}



function add_static_ip # number
{
  num="$1"
  ipaddr=$(addr_value "$num")
  if [ -z "$ipaddr" ]; then
    debug "Skip static IP for number# $num, because it's empty"
    return
  fi

  netmask=$(netmask_value "$num")
  if [ -z "$netmask" ]; then
    error "RS_IP${num}_NETMASK not defined ; Cannot configure static IP address" ${E_NOT_DEFINED_NETMASK}
  fi

  # # optional
  gateway=$(gateway_value "$num")

  device=$(device_name_from_mac "$(mac_value "$num")")

  nameservers=$(nameservers_value "$num")

  output "Setting up static IP address '$ipaddr' for '$device'"
  output "  Netmask: '${netmask}' ; Gateway: '${gateway}'"
  if ! [ -z "$nameservers" ]; then
    output "  Nameservers: '${nameservers}'"
  fi

  is_primary=0
  if [[ -n "$gateway" ]]; then
    is_primary=1
  fi

  msg=$(configure_network_adaptor "static" "$device" "$is_primary" "$ipaddr" "$netmask" "$gateway" "$nameservers")
  error_code=$?

  if ! [ "$error_code" == "0" ]; then
    error "$msg" $error_code
  fi
}

function add_dhcp # number
{
  device=$(device_name_from_mac $(mac_value $1))
  output "Setting up dhcp for '$device'"

  is_primary=0
  if [[ "$1" == "0" ]]; then
    is_primary=1
  fi

  msg=$(configure_network_adaptor "dhcp" "$device" "$is_primary")
  error_code=$?

  if ! [ "$error_code" == "0" ]; then
    error "$msg" $error_code
  fi
}

# This is now quite tricky. We do two routing passes, a pass before the
# system networking is setup (@boot is true) in which we setup static system
# config files and a pass after system networking (@boot is false) in which
# we fix up the remaining routes cases involving DHCP. We have to set any routes
# involving DHCP post networking as we can't know the DHCP gateway beforehand
SEEN_ROUTES_STR=""
function add_route # number, is_init
{
  route=$(route_value $1)

  output "Setting up route '${route}'"
  nat_server_ip=$(nat_server_ip_from "$route")
  cidr=$(cidr_from "$route")
  SEEN_ROUTES_STR=$(seen_route "$cidr" "$nat_server_ip" "$SEEN_ROUTES_STR")
  if [[ $? == 0 ]]; then
    output "Already added route ${cidr} to gateway ${nat_server_ip}, skipping adding it."
    return;
  fi
  debug "NAT Server IP: ${nat_server_ip} ; CIDR: ${cidr}"

  if ! valid_ipv4_cidr "$cidr"; then
    echo "Invalid CIDR: '${cidr}'"; return ${E_INVALID_CIDR}
  fi

  if ! validate_ip4 "$nat_server_ip"; then
    echo "Invalid NAT Server IP: '${nat_server_ip}'"; return ${E_INVALID_NAT_SERVER_IPV4}
  fi


  if [[ "$is_init" != "1" ]]; then
    if network_route_exists "$cidr" "$nat_server_ip"; then
      debug "Skipping route $cidr to $nat_server_ip, already added"
      return 0
    else
      msg=$($CMD_ROUTE_ADD "$cidr" via "$nat_server_ip" 2>&1)
      error_code=$?
      if ! [ "$error_code" == "0" ]; then
        # Same route may show up twice for different interfaces -- treat as non-fatal
        if [[ $msg =~ "File exists" ]]; then
          echo "WARNING: got $msg for $cidr via $nat_server_ip. Route already added to another interface."
        # Static interface may not be up yet and we'll get this error message.  This is acceptable as the routes
        # get written to file for static interfaces, so will get populated that way.
        elif [[ $msg =~ "No such process" ]] || [[ $msg =~ "Network is unreachable" ]]; then
          echo "WARNING: got $msg for $cidr via $nat_server_ip. Interface not ready yet, but should get the route when it's brought up."
        else
          echo "WARNING: unknown error: Adding route $nat_server_ip: $msg"
        fi
      fi
    fi
  fi

  device=$(route_device "$nat_server_ip" "$is_init")
  if [ -n "$device" ]; then
    write_route_config "$device" "$(config_data_route "$cidr" "$nat_server_ip")"
  else
    if [[ "$is_init" == "1" ]]; then
      echo "WARNING: Unable to find appropriate device for $nat_server_ip. Will try again post-DHCP."
    else
      echo "Unable to find appropriate device for $nat_server_ip in post-DHCP phase." ${E_NOT_FOUND_ROUTE_DEVICE}
      return 1
    fi
  fi
}

function from_device_config() {
  local device=$1
  local key=$2
  config_data=$($CMD_IP_SHOW $device 2>&1)
  re=" $key ([^ ]+) "
  if [[ "$config_data" =~ $re ]]; then
    echo "${BASH_REMATCH[1]}"
  else
    echo ""
  fi
}

function add_gateway_route # gateway
{
  output "Adding default gateway route '${1}'"
  msg=$($CMD_ROUTE_ADD_DEFAULT_GATEWAY "$1" 2>&1)
  error_code=$?
  if ! [ "$error_code" == "0" ]; then
    error "$msg" $error_code
  fi
}

function delete_gateway_route # route
{
  msg=$($CMD_ROUTE_DELETE_DEFAULT_GATEWAY $1 2>&1)
  if ! [ $? == 0 ]; then
    echo $msg;
    return 1
  fi
}

function static_ip_numerals
{
  echo $(compgen -v | grep -e 'RS_IP[0-9]*_ADDR' | sort | sed "s/[^0-9]//g")
}

function dhcp_ip_assigment_numerals
{
  assignments=$(compgen -v | grep -e 'RS_IP[0-9]*_ASSIGNMENT')
  dhcp_assignments=$(for assignment in $assignments; do
    if [[ "${!assignment}" == "dhcp" ]]; then echo $assignment; fi
  done)
  echo $(IFS=" "; echo $dhcp_assignments | sort | sed "s/[^0-9]//g")
}

function route_numerals
{
  echo $(compgen -v | grep -e 'RS_ROUTE[0-9]*' | sort | sed "s/[^0-9]//g")
}

function metadata_gateway_value
{
  num=$(compgen -v | grep -e 'RS_IP[0-9]*_GATEWAY' | sort | sed "s/[^0-9]//g" | head -1)
  if ! [ -z $num ]; then
    echo $(gateway_value "$num");
  fi
}

function default_gateway_value
{
  gateway=$($CMD_ROUTE_DEFAULT_GATEWAY | cut -d' ' -f3)
  echo $gateway
}


# setup devices
function setup_devices
{
  debug "Setup devices with provided vSphere metadata"
  for num in $(static_ip_numerals); do
    add_static_ip $num
  done

  for num in $(dhcp_ip_assigment_numerals); do
    add_dhcp $num
  done
}

# setup routes
#  is_init: 1 (true) or 0 (false). If we're in the pre-DHCP or post-DHCP phase. In they pre-DHCP
#           phase we tolerate certain errors as not all devices may be setup yet.
function setup_routes # is_init
{
  local is_init=${1:-0}
  debug "Setup routes with provided vSphere metadata"
  for num in $(route_numerals); do
    try=0
    until [[ $try -ge 24 ]]; do
      try=$((try+1))
      add_route $num $is_init && break
      output "Error adding route. This could be caused by a misconfigured route in the RCA-V adaptor."
      sleep 5
    done
  done
}

function set_default_gateway
{
  metadata_gateway=$(metadata_gateway_value)
  if [ -z "$metadata_gateway" ]; then return; fi
  default_gateway=$(default_gateway_value)
  if [[ "$default_gateway" == "$metadata_gateway" ]]; then return; fi

  output "Setup gateway with provided vSphere metadata value: '${metadata_gateway}'"
  if ! [ -z $default_gateway ]; then
    output "Deleting default gateway route: '${default_gateway}'"
    msg=$(delete_gateway_route "$default_gateway")
    if ! [ $? == 0 ]; then
      echo "Could not delete default route: $msg"
    fi
  fi
  add_gateway_route "$metadata_gateway"
}

function on_init
{
  output "Going to initialize network"
  wait_for_metadata
  setup_devices
  setup_routes 1
  output "Network initialization finished"
}

function on_routes
{
  output "Going to setup routes"
  wait_for_metadata
  setup_routes 0
  set_default_gateway
  output "Routes setup finished"
}

# Verifies that we are running
# under vSphere cloud
function on_vsphere_cloud
{
  if which lspci >/dev/null 2>&1; then
    # This /var/run/rightscale_on_vsphere is a bit of a dirty hack for centos .
    # We run this script twice, the first time during init scripts and second as a hook
    # for the dhcp service when its up. The second time we have far more limited
    # permissions due to selinux dhcpc_t context so lspci doesn't work as it read files
    # in /usr/share/hwdata/pci.ids. rely on cached value form the first run
    if [ ! -e /var/run/rightscale_on_vsphere ]; then
      if ! lspci | grep -qi vmware; then return 1; fi
    fi
    touch /var/run/rightscale_on_vsphere
  fi
  vmtoolsd_bin_path=$(which $VMTOOLSD_NAME)
  if ! [ $? == 0 ]; then return 1; fi
  echo $vmtoolsd_bin_path
}

### START HERE ###

if [[ -z "$CURRENT_DISTRO" ]]; then
  if [ -e /etc/sysconfig/network-scripts ]; then
    CURRENT_DISTRO="centos"
  elif [ -d /etc/network ]; then
    CURRENT_DISTRO="ubuntu"
  else
    error "Current distro is not supported." ${E_UNSUPPORTED_DISTRO}
  fi
fi
debug "Running on ${CURRENT_DISTRO} distro"

if ! [[ ("$1" =~ "--init" ) || ("$1" =~ "--routes") ]] ; then usage; exit ${E_DID_NOTHING}; fi
ACTION="${1:2}"

shift
for opt in "$@"; do
  case $opt in
    "--debug"|"-d" )
      DEBUG=$FLAG_ON
      ;;
  esac
done

VMTOOLSD_BIN=$(on_vsphere_cloud)
if ! [ $? == 0 ]; then
  error "VMware tools are required"
fi
debug "vmtoolsd bin path: $VMTOOLSD_BIN"

on_$ACTION
