#!/bin/bash -e

show_help() {
  cat <<EOT
Usage: ${0##*/} [-options...]
Pulls down RightScale Network Agent from S3 and installs/updates it locally.
  -t                    tarball mode.
  -h                    show this help
  -v                    verbose mode
EOT
}


while getopts "hv" opt; do
  case "$opt" in
    h) show_help; exit 0 ;;
    v) set -x ;;
    '?') show_help >&2; exit 1 ;;
  esac
done

ASSETS_DIR=/usr/local/share/rightnetwork

if which systemctl >/dev/null 2>&1; then
  use_systemctl=0
elif which chkconfig >/dev/null 2>&1; then
  use_chkconfig=0
elif which update-rc.d >/dev/null 2>&1; then
  use_update_rc_d=0
fi

if [ "$use_systemctl" == "0" ]; then
  # CentOS/RHEL 7+ case - Uses systemd to grab pre-networking hook
  cp -f /usr/local/share/rightnetwork/rightscale_network.service /etc/systemd/system/rightscale_network.service
  systemctl daemon-reload
  systemctl enable rightscale_network
elif [ -e /etc/init/networking.conf ]; then
  # Ubuntu 12.04+ case - Use Upstart to grab pre-networking hook
  # CentOS 6 has Upstart installed but with very limited functionality. It doesn't
  # emit events for networking so we can't use the Upstart compatible shim there.
  cp -f /usr/local/share/rightnetwork/rightscale_network.conf /etc/init/rightscale_network.conf
  cp -f /usr/local/share/rightnetwork/rightscale_network-wait.conf /etc/init/rightscale_network-wait.conf
elif [ "$use_update_rc_d" == "0" ]; then
  # Legacy sysvinit for Ubuntu 10.04
  cp -f /usr/local/share/rightnetwork/rightscale_network /etc/init.d/rightscale_network
  /usr/sbin/update-rc.d rightscale_network defaults
elif [ "$use_chkconfig" == "0" ]; then
  # Legacy sysvinit for CentOS/RHEL 5/6
  cp -f /usr/local/share/rightnetwork/rightscale_network /etc/init.d/rightscale_network
  chkconfig --add rightscale_network
else
  echo "Cannot register RightScale network init scripts; no known utility is available"
  exit 1
fi

chmod a+x /usr/local/share/rightnetwork/rightscale_network

if [ -d /etc/dhcp/dhclient-exit-hooks.d ]; then
  # Ubuntu format
  cp -f /usr/local/share/rightnetwork/rightscale_routes /etc/dhcp/dhclient-exit-hooks.d/rightscale_routes
  chmod +x /etc/dhcp/dhclient-exit-hooks.d/rightscale_routes
elif [ -e /etc/dhcp/dhclient.d ]; then
  # CentOS format
  cp -f /usr/local/share/rightnetwork/rightscale_routes.sh /etc/dhcp/dhclient.d/rightscale_routes.sh
  chmod +x /etc/dhcp/dhclient.d/rightscale_routes.sh
  # On CentOS 6/7 with selinux enabled, routes must have the right context to be executed by dhclient
  if which sestatus >/dev/null 2>&1; then
    if sestatus | grep enabled >/dev/null 2>&1; then
      chcon -v --type=bin_t  /etc/dhcp/dhclient.d/rightscale_routes.sh
      # Fixes a bug -- permission denied on some of the stock images!
      restorecon -v /etc/resolv.conf
    fi

    # After a fresh non-package-based install this may have the wrong label
    if ls -Z $(readlink -f $(which vmtoolsd)) | grep lib_t; then
      echo "vmtoolsd is labelled with the wrong selinux context. It should be bin_t but is lib_t."
      echo "For further details see: http://docs.rightscale.com/rl10/reference/rl10_rcav.html#overview-troubleshooting"
      echo "To fix this please run:"
      echo "  restorecon -v $(readlink -f $(which vmtoolsd))"
      echo
      echo "Then re-run this installer"
    fi

    # This is only needed on centos 7 - vmtools is in another context (vmtools_exec_t) from dhclient. This module gives
    # permission for that context to the dhclient (dhcpc_t) context and permission to write a log file. On centos 6
    # vmtools is in the bin_t context, which dhclient can use.
    if ls -Z $(readlink -f $(which vmtoolsd)) | grep vmtools_exec_t >/dev/null 2>&1; then
      # .pp module can be compiled as such:
      # checkmodule -M -m -o $ASSETS_DIR/rightscale_routes.mod $ASSETS_DIR/rightscale_routes.te
      # semodule_package -m $ASSETS_DIR/rightscale_routes.mod -o $ASSETS_DIR/rightscale_routes.pp
      semodule -i $ASSETS_DIR/rightscale_routes.pp
    fi
  fi
fi

chmod +x /usr/local/share/rightnetwork/rightscale_network
chmod +x /usr/local/bin/rightnetwork.sh
if [ -e /etc/init.d/rightscale_network ]; then
  chmod +x /etc/init.d/rightscale_network
fi

# We don't want cloud-init to create conflicting configs in this scenario
if [ -d /etc/cloud/cloud.cfg.d ]; then
  cfg_file=/etc/cloud/cloud.cfg.d/99-rightscale-disable-network-config.cfg
  echo "Disabling cloud-init from managing networking in $cfg_file"
  echo "network: {config: disabled}" > $cfg_file
fi

